-- FUNCTION: public.logisticTrain(regclass, double precision, integer)

-- DROP FUNCTION public."logisticTrain"(regclass, double precision, integer);

CREATE OR REPLACE FUNCTION public."logisticTrain"(
	table_train regclass,
	learning_rate double precision,
	num_epochs integer)
    RETURNS double precision[]
    LANGUAGE 'plpgsql'
    COST 100
    VOLATILE PARALLEL UNSAFE
AS $BODY$
declare
	epoch integer := 1;
	rowLength integer;
	coefficients double precision[];
	row_array double precision[];
	yPredict double precision;
	oneRow public."logRegTest"%rowType;
	i integer := 1;
	errorRate double precision;
begin
	coefficients := array[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0];
	loop
		exit when epoch > num_epochs;
		--raise notice 'epoch %', epoch;
		for oneRow in
			execute format ('select * from %s', table_train)
		loop
			i := 1;
			row_array := array[oneRow.x1, oneRow.x2, oneRow.x3, oneRow.x4, oneRow.x5, oneRow.x6, oneRow.x7, oneRow.x8, oneRow.x9, oneRow.x10, oneRow.x11,oneRow.y];
			yPredict := public."logRegPredict"(row_array, coefficients);
			--raise notice 'yPredict = %', yPredict;
			errorRate := row_array[array_length(row_array, 1)] - yPredict;
			coefficients[1] := coefficients[1] + learning_rate * errorRate * yPredict * (1.0 - yPredict);
			loop
				exit when i = array_length(row_array, 1);
				coefficients[i + 1] := coefficients[i+1] + learning_rate * errorRate * yPredict * (1.0 - yPredict) * row_array[i];
				i := i + 1;
			end loop;
		end loop;
		--raise notice 'coef = %', coefficients;
		epoch := epoch + 1;
	end loop;
	return coefficients;
end;
$BODY$;

ALTER FUNCTION public."logisticTrain"(regclass, double precision, integer)
    OWNER TO postgres;
